using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HarmonyLib;
using TaleWorlds.CampaignSystem;
using TaleWorlds.CampaignSystem.ViewModelCollection.Craft;
using TaleWorlds.CampaignSystem.ViewModelCollection.Craft.WeaponDesign;
using TaleWorlds.Core;
using TaleWorlds.Library;
using TaleWorlds.Localization;
using TaleWorlds.MountAndBlade;

namespace SmithingValue
{
    public class SmithingValueModule: MBSubModuleBase
    {
        protected override void OnSubModuleLoad()
        {
            base.OnSubModuleLoad();
            var harmony = new Harmony("SmithingValue");
            harmony.PatchAll();
        }
    }
}

// Add the value to the property list
[HarmonyPatch(typeof(WeaponDesignVM))]
[HarmonyPatch("RefreshStats")]
class PatchRefreshStats
{
    static AccessTools.FieldRef<WeaponDesignVM, MBBindingList<CraftingListPropertyItem>> primaryPropertyListRef = AccessTools.FieldRefAccess<WeaponDesignVM, MBBindingList<CraftingListPropertyItem>>("_primaryPropertyList");
    static AccessTools.FieldRef<WeaponDesignVM, Crafting> craftingRef = AccessTools.FieldRefAccess<WeaponDesignVM, Crafting>("_crafting");
    static void Postfix(WeaponDesignVM __instance)
    {
        ItemObject weapon = craftingRef(__instance).GetCurrentCraftedItemObject(false, null);
        EquipmentElement equipment = new EquipmentElement(weapon);
        int price = Campaign.Current.Models.TradeItemPriceFactorModel.GetPrice(equipment, Campaign.Current.MainParty, null, true, 0, 0, 0);
        CraftingListPropertyItem valueItem = new CraftingListPropertyItem(new TextObject("Value: ", null), 50000f, (float)price, 0f, CraftingTemplate.CraftingStatTypes.NumStatTypes, false);
        valueItem.IsValidForUsage = true;
        primaryPropertyListRef(__instance).Add(valueItem);
    }
}

// Prevent the value from being in the result screen
[HarmonyPatch(typeof(WeaponDesignVM))]
[HarmonyPatch("UpdateResultPropertyList")]
class PatchUpdateResult
{
    static AccessTools.FieldRef<WeaponDesignVM, MBBindingList<WeaponDesignResultPropertyItemVM>> designResultRef = AccessTools.FieldRefAccess<WeaponDesignVM, MBBindingList<WeaponDesignResultPropertyItemVM>>("_designResultPropertyList");
    static void Postfix(WeaponDesignVM __instance)
    {
        designResultRef(__instance).RemoveAt(designResultRef(__instance).Count - 1);
    }
}

// Fix 1.6.0 naming bug
[HarmonyPatch(typeof(WeaponDesignResultPopupVM))]
[HarmonyPatch("ExecuteFinalizeCrafting")]
class PatchWeaponName
{
    static AccessTools.FieldRef<WeaponDesignResultPopupVM, Crafting> craftingRef = AccessTools.FieldRefAccess<WeaponDesignResultPopupVM, Crafting>("_crafting");
    static void Prefix(WeaponDesignResultPopupVM __instance)
    {
        craftingRef(__instance).CraftedWeaponName = __instance.ItemName;
    }
}